/*
 * (c) Copyright 2003 M. van Meegen
 * All Rights Reserved.
 * Sourcecode is licensed under Mozilla Public License 1.1
 */
package edu.virginia.cs.escjava;
import java.io.File;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;

/**
 * watches for changes requiring checkstyle to run
 * and runs the adequate checker on the adequate resources.
 * For failed file checks warning markers are set.
 * 
 * @author Marco van Meegen
 */
public class EscjavaBuilder extends IncrementalProjectBuilder {

	/** suffix for files to check */
	private static final String JAVA_SUFFIX = "java";

	/**
	 * Constructor for CheckclipseBuilder.
	 */
	public EscjavaBuilder() {
		super();
	}

	/**
	 * implements the checkstyle process
	 * @see InternalBuilder#build(int, Map, IProgressMonitor)
	 */
	protected IProject[] build(int kind, Map args, IProgressMonitor monitor) throws CoreException {
		if (kind == IncrementalProjectBuilder.FULL_BUILD) {
			fullBuild(monitor);
		} else {
			IResourceDelta delta = getDelta(getProject());
			if (delta == null) {
				fullBuild(monitor);
			} else {
				incrementalBuild(delta, monitor);
			}
		}

		return null;
	}

	/**
	 * checks the style for the given file
	 * @param r resource to check (must be a file with suffix java)
	 * @param m progress monitor to report to
	 */
	protected void checkStyleForFile(IResource r, IProgressMonitor m) {
		if (isApplicable(r)) {
			m.subTask("Analyzing file " + r.getName());
			System.out.println("Checking " + r.getName() + " in project " +
				r.getProject().getName());
			
			String classpathString = ".";
				
			try {

//			IJavaModel theJavaModel = JavaCore.create(EscjavaPlugin.getWorkspace().getRoot());
			IJavaProject ijp = JavaCore.create(r.getProject());
			IClasspathEntry[] classpaths = ijp.getResolvedClasspath(true);
			IWorkspaceRoot root= ResourcesPlugin.getWorkspace().getRoot();

			
			for(int i = 0; i < classpaths.length; i++)
			{
				IResource cpResource = root.findMember(classpaths[i].getPath().toOSString());
				if (cpResource != null) {
					classpathString = classpathString + ";" + cpResource.getLocation().toOSString();
					//System.out.println(i + " " + " resolved: " + cpResource.getLocation().toOSString());
				}
				else
				{
					classpathString = classpathString + ";" + classpaths[i].getPath().toOSString();
					//System.out.println(i + " " + classpaths[i].getPath().toOSString());
				}
			}
			
		//	classpathString = "\"" + classpathString + "\"";
			
			System.out.println("Java project name: " + ijp.getProject().getName());
			} catch (Exception ex)
			{
				System.out.println("Caught exception " + ex.toString());
				ex.printStackTrace();
			}
			System.out.println("no exception");
			Checker checker = null;
			try {
				// remove all error markers
				r.deleteMarkers(EscjavaPlugin.CHECKCLIPSE_MARKER_ID, true, IResource.DEPTH_ZERO);

				// if filter is active, ignore file
				//if (EscjavaConfiguration.getEscjavaConfiguration(r.getProject()).isApplicable(r)) { 
					
					// get a configured checker for the project the file belongs to
					EscjavaErrorHandler handler = new EscjavaErrorHandler(r);
					checker = EscjavaConfiguration.getChecker(r.getProject());
					checker.addListener(handler);
	
					// now process the file resource
					File f = r.getLocation().toFile();
					File[] files = new File[] {
						f
					};
					
					checker.process(files, classpathString);
					checker.destroy();
				//} 
			} catch (Throwable ex) {
				EscjavaPlugin.log("Exception occured while checking style for " + r.getName(), ex);
			} finally {
				if (checker != null) {
					// ATTENTION: does not destroy the checker,
					// but removes all handlers. There is no other
					// way to remove listeners !
					checker.destroy();
				}
			}
		}
	}


	/** checks if the given resource is applicable for checkstyle.
	 * Currently checks if it is a file which exists and has
	 * suffix java
	 * @param r resource to check
	 * @return true, if applicable
	 */
	private boolean isApplicable(IResource r) {
		if (!(r instanceof IFile)) {
			return false;
		}
		IFile f = (IFile) r;

		return f.exists() && f.getFileExtension() != null && f.getFileExtension().equals(JAVA_SUFFIX);
	}

	/**
	 * does a full build of the project. 
	 * @param monitor monitor
	 * @throws CoreException core exception
	 */
	protected void fullBuild(final IProgressMonitor monitor) throws CoreException {
		// create visitor and call the standard checkStyleForFile method
		// flush all cached config files to make sure, a full build
		// reflects config changes
		EscjavaConfiguration.invalidate();
		getProject().accept(new IResourceVisitor() {
			public boolean visit(IResource r) {
				checkStyleForFile(r, monitor);
				return true;
			}

		});
	}

	/**
	 * do incremental build; only one file is passed to checkstyle.
	 * TODO: ignore fileset-tests
	 * @param delta change-delta
	 * @param monitor monitor
	 * @throws CoreException core exception
	 */
	protected void incrementalBuild(IResourceDelta delta, final IProgressMonitor monitor) throws CoreException {
		// create visitor and call the standard buildGrammarFile method	
		delta.accept(new IResourceDeltaVisitor() {
			public boolean visit(IResourceDelta delta) {
				switch (delta.getKind()) {
					case IResourceDelta.CHANGED :
						// handle changed resource
						// fall through ...
					case IResourceDelta.ADDED :
						// handle added or changed resource
						checkStyleForFile(delta.getResource(), monitor);
						break;
					case IResourceDelta.REMOVED :
						// handle removed resource
						// nothing to do, since markers are removed by framework
						break;
				}
				return true;
			}

		});
	}
}