/*
 * (c) Copyright 2002 M. van Meegen
 * All Rights Reserved.
 * Sourcecode is licensed under Mozilla Public License 1.1
 */
package edu.virginia.cs.escjava;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.runtime.CoreException;

/**
 * Checkstyle Nature to automagically install builder
 * for projects shared in a team.
 * @author Marco van Meegen
 */
public class EscjavaNature implements IProjectNature {

	/** project this nature is associated with */
	private IProject project;

	/**
	 * configures the nature by adding the associated builder
	 * @see org.eclipse.core.resources.IProjectNature#configure()
	 */
	public void configure() throws CoreException {
		addToBuildSpec(EscjavaPlugin.CHECKCLIPSE_BUILDER_ID, project);
	}
	
	/**
	 *  deconfigures nature by removing associated builder
	 * @see org.eclipse.core.resources.IProjectNature#deconfigure()
	 */
	public void deconfigure() throws CoreException {
		EscjavaNature.removeFromBuildSpec(EscjavaPlugin.CHECKCLIPSE_BUILDER_ID, project);
	}
	
	/** @return associated project */
	public IProject getProject() {
		return project;
	}
	
	/** @param value sets associated project */
	public void setProject(IProject value) {
		project = value;
	}


	/** adds the given builder to/from the project.
	 * if already added, no-op and false is returned.
	 * @param builderId builder id to add
	 * @param project project to add builder to
	 * @return true if added; false if builder was already registered
	 * @throws CoreException if builder could not be added
	 */
	protected static boolean addToBuildSpec(String builderId, IProject project) throws CoreException {
		
		System.out.println("addToBuildSpec called");
	
		IProjectDescription description = project.getDescription();
		ICommand[] commands = description.getBuildSpec();
		int index = -1;
	
		for (int i = 0; i < commands.length; ++i) {
			if (commands[i].getBuilderName().equals(builderId)) {
				index = i;
			}
		}
	
		// not there --> add it
		if (index == -1) {
			// not found, register
			ICommand[] newCommands = new ICommand[commands.length + 1];
			System.arraycopy(commands, 0, newCommands, 1, commands.length);
			newCommands[0] = description.newCommand();
			newCommands[0].setBuilderName(builderId);
			description.setBuildSpec(newCommands);
			project.setDescription(description, null);
		}
		
		return index == -1;
	}

	/**
	 * removes builder from project
	 * @param builderId builder to remove
	 * @param project project
	 * @return true, if builder was found and removed, false if not found
	 * @throws CoreException if something went wrong
	 */
	protected static boolean removeFromBuildSpec(String builderId, IProject project) throws CoreException {
		IProjectDescription description = project.getDescription();
		ICommand[] commands = description.getBuildSpec();
		
		for (int i = 0; i < commands.length; ++i) {
			if (commands[i].getBuilderName().equals(builderId)) {
				ICommand[] newCommands = new ICommand[commands.length - 1];
				System.arraycopy(commands, 0, newCommands, 0, i);
				System.arraycopy(commands, i + 1, newCommands, i, commands.length - i - 1);
				description.setBuildSpec(newCommands);
				project.setDescription(description, null);
				return true;
			}
		}
		return false;
	}
	

	/** adds this nature to the project.
	 * if already added, no-op and false is returned.
	 * @param project project to add nature to
	 * @return true if added; false if nature was already registered
	 * @throws CoreException if builder could not be added
	 */
	public static boolean add(IProject project) throws CoreException {
	
		String natureId = EscjavaPlugin.CHECKCLIPSE_NATURE_ID;
		IProjectDescription description = project.getDescription();
		String[] natureIds = description.getNatureIds();

		// not there --> add it
		if (!description.hasNature(natureId))
		{
			// not found, register
			String[] newNatureIds = new String[natureIds.length + 1];
			System.arraycopy(natureIds, 0, newNatureIds, 0, natureIds.length);
			newNatureIds[natureIds.length] = natureId;
			description.setNatureIds(newNatureIds);
			String[] natureids = description.getNatureIds();
			project.setDescription(description, null);
			return true;
		}
		else
		{
			project.setDescription(description, null);
			return false;
		}
	}

	/**
	 * removes nature from project
	 * @param project project
	 * @return true, if nature was found and removed, false if not found
	 * @throws CoreException if something went wrong
	 */
	public static boolean remove(IProject project) throws CoreException {
		String natureId = EscjavaPlugin.CHECKCLIPSE_NATURE_ID;
		IProjectDescription description = project.getDescription();
		String[] natureIds = description.getNatureIds();
		
		for (int i = 0; i < natureIds.length; ++i) {
			if (natureIds[i].equals(natureId)) {
				String[] newNatureIds = new String[natureIds.length - 1];
				System.arraycopy(natureIds, 0, newNatureIds, 0, i);
				System.arraycopy(natureIds, i + 1, newNatureIds, i, natureIds.length - i - 1);
				description.setNatureIds(newNatureIds);
				project.setDescription(description, null);
				return true;
			}
		}
		return false;
	}


}
